﻿## no critic qw(ProhibitMagicNumbers RequireInterpolationOfMetachars ProhibitLongLines)
use Test::More tests => 190;
use strict;
use warnings;
use DE_EPAGES::Core::API::Constants qw( CALLER_SUBNAME );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::TestParameter qw( TestObject TestString TestIpAddress TestInteger
                                            TestDateTime TestDomain TestUrl TestIBAN TestBIC );

sub _RequireString {
    my ($Param1, $MinLength, $MaxLength) = @_;
    TestString( 'Param1', $Param1, $MinLength, $MaxLength );
}

sub _TestString {

    _TestNoError(
        sub { TestString('Name', 'String'); },
        'TestString(\'Name\', \'String\')'
    );

    _TestError(
        sub { TestString(undef, undef); },
        'TestString(undef)',
        'PARAMETER_MISSING',
        'Missing parameter Name in call to DE_EPAGES::Core::API::TestParameter::TestString'
    );

    _TestError(
        sub { _RequireString(undef); },
        '_RequireString(undef)',
        'PARAMETER_MISSING',
        'Missing parameter Param1 in call to main::_RequireString'
    );

    _TestError(
        sub { _RequireString(''); },
        '_RequireString(\'\')',
        'STRING_EMPTY',
        'Param1 is empty in call to main::_RequireString'
    );

    _TestError(
        sub { _RequireString('   '); },
        '_RequireString(\'   \')',
        'STRING_EMPTY',
        'Param1 is empty in call to main::_RequireString'
    );

    _TestError(
        sub { _RequireString('too short', 10); },
        '_RequireString(\'too short\', 10)',
        'STRING_TOO_SHORT',
        'Param1 is too short (length(\'too short\') < 10) in call to main::_RequireString'
    );

    _TestError(
        sub { _RequireString('too long', undef, 3); },
        '_RequireString(\'too long\', undef, 3)',
        'STRING_TOO_LONG',
        'Param1 is too long (length(\'too long\') > 3) in call to main::_RequireString'
    );

    use utf8;
    my $Uft8String = 'Радуга';
    no utf8;

    eval { TestString('Name', $Uft8String, undef, 5); };
    ok( ExistsError() && GetError->code eq 'STRING_TOO_LONG', 'utf8 string is too long' );

    eval { TestString('Name', $Uft8String, 7, undef); };
    ok( ExistsError() && GetError->code eq 'STRING_TOO_SHORT', 'utf8 string is too short' );

    eval { TestString('Name', $Uft8String, 6, 6); };
    ok( !ExistsError(), 'utf8 string is ok' );

    _TestError(
        sub { _RequireString('Value', 'ABC') },
        '_RequireString(\'Value\', \'ABC\')',
        'FORMAT_NOT_INTEGER',
        'MinLength is no integer number (ABC) in call to DE_EPAGES::Core::API::TestParameter::TestString',
        'main::_RequireString'
    );

    _TestError(
        sub { _RequireString([]) },
        '_RequireString([])',
        'STRING_ISOBJECT',
        'Param1 is an object in call to main::_RequireString',
        'main::__ANON__'
    );
}

sub _TestDateTime {

    _TestError(
        sub { TestDateTime('Param1', {}) },
        'TestDateTime(\'Param1\', {})',
        'STRING_ISOBJECT',
        'Param1 is an object in call to main::__ANON__',
        '(eval)'
    );
}

sub _RequireObject {
    my ($Param1, $BaseClass) = @_;
    TestObject( 'Param1', $Param1, $BaseClass );
}

sub _TestObject {

    _TestError(
        sub { TestObject(undef, undef); },
        '_RequireObject(undef)',
        'PARAMETER_MISSING',
        'Missing parameter Name in call to DE_EPAGES::Core::API::TestParameter::TestObject'
    );

    _TestError(
        sub { _RequireObject(undef); },
        '_RequireObject(undef)',
        'PARAMETER_MISSING',
        'Missing parameter Param1 in call to main::_RequireObject'
    );

    _TestError(
        sub { _RequireObject('String'); },
        '_RequireObject(\'String\')',
        'PARAMETER_NO_OBJECT',
        'Param1 is not an object () in call to main::_RequireObject'
    );

    _TestError(
        sub { _RequireObject([]) },
        '_RequireObject([])',
        'PARAMETER_NO_OBJECT',
        'Param1 is not an object (ARRAY) in call to main::_RequireObject'
    );

    my $Object = MY::TestPackage->new;
    _TestNoError(
        sub { TestObject('Name', $Object); },
        'TestObject(\'Name\', \$Object)'
    );

    _TestNoError(
        sub { _RequireObject($Object, 'MY::TestPackage') },
        '_RequireObject(\$Object, \'MY::TestPackage\')'
    );

    _TestNoError(
        sub { _RequireObject($Object, 'BasePackage') },
        '_RequireObject($Object, \'BasePackage\')',
    );

    _TestError(
        sub { _RequireObject($Object, 'WrongPackage') },
        '_RequireObject($Object, \'WrongPackage\')',
        'WRONG_CLASS',
        'Param1 is not an instance of WrongPackage (MY::TestPackage) in call to main::_RequireObject'
    );
}

sub _TestCallStack {

    my $Object = MY::TestPackage->new;

    # test error position and call stack
    _TestError(
        sub { $Object->missingParameter; },
        'missingParameter',
        'PARAMETER_MISSING',
        'Missing parameter Param1 in call to BasePackage::missingParameter',
        'MY::TestPackage::missingParameter'
    );

    _TestError(
        sub { $Object->invalidInteger; },
        'invalidInteger',
        'FORMAT_NOT_INTEGER',
        'Param1 is no integer number (ABC) in call to BasePackage::invalidInteger',
        'MY::TestPackage::invalidInteger'
    );
}

sub _RequireInteger {
    my ($Param1) = @_;
    TestInteger( 'Param1', $Param1, undef, 1000 );
}

sub _TestInteger {

    _TestError(
        sub { TestInteger('Param1', 123, 'ABC') },
        'TestInteger(\'Param1\', 123, \'ABC\')',
        'FORMAT_NOT_INTEGER',
        'MinValue is no integer number (ABC) in call to DE_EPAGES::Core::API::TestParameter::TestInteger',
        'main::__ANON__'
    );

    _TestError(
        sub { _RequireInteger(-111111111123) },
        '_RequireInteger(-111111111123)',
        'INTEGER_TOO_SMALL',
        'Param1 is too small (-111111111123 < -2147483648) in call to main::_RequireInteger',
        'main::__ANON__'
    );

    _TestError(
        sub { _RequireInteger(2000) },
        '_RequireInteger(2000)',
        'INTEGER_TOO_LARGE',
        'Param1 is too large (2000 > 1000) in call to main::_RequireInteger',
        'main::__ANON__'
    );
}

sub _RequireIpAddress {
    my ($Param1) = @_;
    TestIpAddress( 'Param1', $Param1 );
}

sub _TestIpAddress {

    my %IpTests = (
        'foo'                 => 'Param1 (foo) has wrong format (VOID_IP_ADDRESS)',
        'foo.bar'             => 'Param1 (foo.bar) has wrong format (VOID_IP_ADDRESS)',
        '1.2.3.4.5'           => 'Param1 (1.2.3.4.5) has wrong format (VOID_IPv4_ADDRESS)',
        '123.234.156.103'     => '',
        '123.234.256.103'     => 'Param1 (123.234.256.103) has wrong format (VOID_IPv4_ADDRESS)',
        '::1'                 => '',
        '1:2:3'               => 'Param1 (1:2:3) has wrong format (VOID_IPv6_ADDRESS)',
        '::ffff:1.2.3.4'      => '',
        '1:2:3:4:5:6:1.2.3.4' => 'Param1 (1:2:3:4:5:6:1.2.3.4) has wrong format (VOID_IPv6_ADDRESS)',
        '::ffff:1.2.3'        => 'Param1 (::ffff:1.2.3) has wrong format (VOID_IPv6_ADDRESS)',
        '1:2:3:4:5:6:7:8'     => '',
        '1:2:3:4:5:6:7:x'     => 'Param1 (1:2:3:4:5:6:7:x) has wrong format (VOID_IP_ADDRESS)',
        undef                 => 'Param1 (undef) has wrong format (VOID_IP_ADDRESS)',
    );

    foreach my $IpAddress (keys %IpTests) {
        my $ShortMessage = $IpTests{$IpAddress};
        if( $ShortMessage eq '' ) {
            _TestNoError(
                sub { _RequireIpAddress($IpAddress); },
                "_RequireIpAddress($IpAddress)"
            );
        }
        else {
            _TestError(
                sub { _RequireIpAddress($IpAddress); },
                "_RequireIpAddress($IpAddress)",
                'FORMAT_NOT_IPADDRESS',
                $ShortMessage . ' in call to main::_RequireIpAddress'
            );
        }
    }
}

sub _RequireDomain {
    my ($Param1) = @_;
    TestDomain( 'Param1', $Param1 );
}

sub _TestDomain {

    my %DomainTests = (
        'foo.bar'       => '',
        '0.1.2.3'       => '',
        '123reg.co.uk'  => '',
        '-epages.de'    => 'Param1 (-epages.de) has wrong format (DOMAIN_BOUNDARY_HYPHEN)',
        'www.l-.pt'     => 'Param1 (www.l-.pt) has wrong format (DOMAIN_BOUNDARY_HYPHEN)',
        'www..pt'       => 'Param1 (www..pt) has wrong format (DOMAIN_ATOM_EMPTY)',
        'epages_.de'    => 'Param1 (epages_.de) has wrong format (DOMAIN_CHARS_VOID)',
        'm\x{FC}ller.de'=> 'Param1 (m\x{FC}ller.de) has wrong format (DOMAIN_CHARS_VOID)',
        'EightMegabytesAndConstantlySwappingEmacsManualsAreCrypticandSurreal'
                        => 'Param1 (EightMegabytesAndConstantlySwappingEmacsManualsAreCrypticandSurreal)'
                           .' has wrong format (DOMAIN_ATOM_GT_63_CHARS)',
        'Each.Manuals.Audience.is.Completely.Stupified.Emacs.Means.A.Crappy.Screen.Eventually.Munches.All'
            .'.Computer.Storage.Even.My.Aunt.Crashes.the.System.Eradication.of.Memory.Accomplished.with'
            .'.Complete.Simplicity.Elsewhere.Maybe.Alternative.Civilizations.Survive'
                        => 'Param1 (Each.Manuals.Audience.is.Completely.Stupified.Emacs.Means.A.Crappy.Screen'
                           .'.Eventually.Munches.All.Computer.Storage.Even.My.Aunt.Crashes.the.System'
                           .'.Eradication.of.Memory.Accomplished.with.Complete.Simplicity.Elsewhere.Maybe'
                           .'.Alternative.Civilizations.Survive) has wrong format (DOMAIN_NAME_GT_253_CHARS)',
    );

    foreach my $Domain (keys %DomainTests) {
        my $ShortMessage = $DomainTests{$Domain};
        if( $ShortMessage eq '' ) {
            _TestNoError(
                sub { _RequireDomain($Domain); },
                "_RequireDomain($Domain)"
            );
        }
        else {
            _TestError(
                sub { _RequireDomain($Domain); },
                "_RequireDomain($Domain)",
                'FORMAT_NOT_DOMAIN',
                $ShortMessage . ' in call to main::_RequireDomain'
            );
        }
    }
}

sub _RequireUrl {
    my ($Param1) = @_;
    TestUrl( 'Param1', $Param1 );
}

sub _TestUrl {

    my %UrlTests = (
        'xyz://.~%ff!$&()*+,;:=-@[::1]:888?xyz' => '',
        'xyz://user:password@[::1]:8888/xyz'    => '',
        'xyz://user:password@[::1]:8888#xyz'    => '',
        'xyz://example.com&story=news@10.0.0.1' => '',
        'xyz:/user:password@[::1]:8888/xyz'     => 'Param1 (xyz:/user:password@[::1]:8888/xyz) has wrong format (URL_FORMAT_VOID)',
        'xyz://user:pass@word@[::1]:8888/xyz'   => 'Param1 (xyz://user:pass@word@[::1]:8888/xyz) has wrong format (URL_USERINFO_VOID_CHARS)',
        'xyz://user:password@m\x{FC}ller.de/xyz/' => 'Param1 (xyz://user:password@m\x{FC}ller.de/xyz/) has wrong format (DOMAIN_CHARS_VOID)',
        'xyz://user:password@-epages.de/xyz/'   => 'Param1 (xyz://user:password@-epages.de/xyz/) has wrong format (DOMAIN_BOUNDARY_HYPHEN)',
        'xyz://user%xx@epages.de/xyz/'          => 'Param1 (xyz://user%xx@epages.de/xyz/) has wrong format (URL_USERINFO_VOID_%HEX)',
        'xyz://m\x{FC}ller@epages.de/xyz/'      => 'Param1 (xyz://m\x{FC}ller@epages.de/xyz/) has wrong format (URL_USERINFO_VOID_CHARS)',
    );

    foreach my $Url (keys %UrlTests) {
        my $ShortMessage = $UrlTests{$Url};
        if( $ShortMessage eq '' ) {
            _TestNoError(
                sub { _RequireUrl($Url); },
                "_RequireUrl($Url)"
            );
        }
        else {
            _TestError(
                sub { _RequireUrl($Url); },
                "_RequireUrl($Url)",
                'FORMAT_NOT_URL',
                $ShortMessage . ' in call to main::_RequireUrl'
            );
        }
    }
}

sub _RequireBIC {
    my ($Param1) = @_;
    TestBIC( 'Param1', $Param1 );
}

sub _TestBIC {

    my %BICTests = (
		'RBOSGGSX'       => '',
		'GENODEF1JEV'     => '',
		'CEDELULLXXX'     => '',
		'UBSWCHZH80A'     => '',
		'UWSWCHZA80A'     => '',

		'UBSWCHZH80'      => 'Param1 (UBSWCHZH80) is not a valid BIC (BIC_WRONG_LENGTH)',
		'U1SWCHZH80A'     => 'Param1 (U1SWCHZH80A) is not a valid BIC (BIC_WRONG_BANKCODE)',
		'UASWCH0H80A'     => 'Param1 (UASWCH0H80A) is not a valid BIC (BIC_WRONG_TOWNCODE)',
		'UCSWCH1H80A'     => 'Param1 (UCSWCH1H80A) is not a valid BIC (BIC_WRONG_TOWNCODE)',
		'UCSWCH2O80A'     => 'Param1 (UCSWCH2O80A) is not a valid BIC (BIC_WRONG_TOWNCODE)',

    );

    foreach my $BIC (keys %BICTests) {
        my $ShortMessage = $BICTests{$BIC};
        if( $ShortMessage eq '' ) {
            _TestNoError(
                sub { _RequireBIC($BIC); },
                "_RequireBIC($BIC)"
            );
        }
        else {
            _TestError(
                sub { _RequireBIC($BIC); },
                "_RequireBIC($BIC)",
                'FORMAT_NOT_BIC',
                $ShortMessage . ' in call to DE_EPAGES::Core::API::TestParameter::TestBIC'
            );
        }
    }

}


sub _RequireIBAN {
    my ($Param1) = @_;
    TestIBAN( 'Param1', $Param1, 1 );
}

sub _TestIBAN {

    my %IBANTests = (
        'FR1420041010050500013M02606'     => '',
        'DE89370400440532013000'          => '',
		'MT84MALT011000012345MTLCAST001S' => '',
		'IL620108000000099999999'         => '',
		'DE 89 37040044 0532013000'       => '',
		'DE89 37040044    0532013000'     => '',
		'90 37040044 0532013000'       => 'Param1 (90 37040044 0532013000) is not a valid IBAN (IBAN_NO_ISO_CODE)',
        'DE 90 37040044 0532013000'    => 'Param1 (DE 90 37040044 0532013000) is not a valid IBAN (IBAN_MOD_TEST_FAILED)',
		'DE 89 97040044 0532013000'    => 'Param1 (DE 89 97040044 0532013000) is not a valid IBAN (IBAN_MOD_TEST_FAILED)',
		'DE 89 37040044 0532013005'    => 'Param1 (DE 89 37040044 0532013005) is not a valid IBAN (IBAN_MOD_TEST_FAILED)',
		'DE 89 37040044 0532013005234234d345g72'    => 'Param1 (DE 89 37040044 0532013005234234d345g72) is not a valid IBAN (IBAN_WRONG_LENGTH)',

    );

    foreach my $IBAN (keys %IBANTests) {
        my $ShortMessage = $IBANTests{$IBAN};
        if( $ShortMessage eq '' ) {
            _TestNoError(
                sub { _RequireIBAN($IBAN); },
                "_RequireIBAN($IBAN)"
            );
        }
        else {
            _TestError(
                sub { _RequireIBAN($IBAN); },
                "_RequireIBAN($IBAN)",
                'FORMAT_NOT_IBAN',
                $ShortMessage . ' in call to DE_EPAGES::Core::API::TestParameter::TestIBAN'
            );
        }
    }

}

sub _TestError {
    my ($Sub, $TestName, $Code, $ShortMessage, $CallerSub) = @_;

    eval { $Sub->(); };
    my $ExistsError = ExistsError();
    my $Error = GetError();
    ok( $ExistsError, "$TestName: ExistsError" );
    SKIP: {
        skip 'no error occured', 1 unless $ExistsError;
        is( $Error->code, $Code, "$TestName: code=$Code" );
    };
    if( defined $ShortMessage ) {
        SKIP: {
            skip 'no error occured', 1 unless $ExistsError;
            is( $Error->shortMessage, $ShortMessage, "$TestName: shortMessage" )
        };
    }
    if( defined $CallerSub ) {
        SKIP: {
            skip 'no error occured', 1 unless $ExistsError;
            is( $Error->callers->[1]->[CALLER_SUBNAME], $CallerSub, "$TestName: caller" ) ;
        }
    }
}

sub _TestNoError {
    my ($Sub, $TestName) = @_;

    eval { $Sub->(); };
    if( ExistsError() ) {
        ok( 0, "$TestName: " . GetError()->longMessage );
    }
    else {
        ok( 1, "$TestName: no error" );
    }
}

_TestString();
_TestDateTime();
_TestObject();
_TestCallStack();
_TestInteger();
_TestIpAddress();
_TestDomain();
_TestUrl();
_TestIBAN();
_TestBIC();

#==============================================================================
package BasePackage;

use strict;
use DE_EPAGES::Core::API::TestParameter qw( TestInteger TestString );

sub new {
    my $class = shift;
    my @Params = @_;
    return bless { @Params }, $class;
}

sub missingParameter {
    TestString('Param1', undef);
}

sub invalidInteger {
    TestInteger('Param1', 'ABC');
}

#==============================================================================
package MY::TestPackage;            ## no critic qw( ProhibitMultiplePackages )
our @ISA;                           ## no critic qw( ProhibitExplicitISA )
BEGIN { @ISA = qw( BasePackage ); } ## no critic qw( ProhibitExplicitISA )

use strict;

sub new {
    my $class = shift;
    my @Params = @_;
    return bless { @Params }, $class;
}

sub missingParameter {
    shift->SUPER::missingParameter;
}

sub invalidInteger {
    shift->SUPER::invalidInteger;
}
